A.12 Cortando Samples

# Cortando Samples

En el episodio 3 de esta serie sobre Sonic Pi vimos como hace loop, alargar y filtrar uno de los más famosos ritmos de batería de todos los tiempos - el ritmo Amen. En este tutorial vamos a llevar esto un paso más allá y aprenderemos como cortarlo en trozos, reordenar los trozos, y pegarlos juntos de nuevo en un orden completamente diferente. Si esto te suena un poco a locura, no te preocupes, pronto será más claro y tu controlarás una nueva y potente herramienta para manejar tus sesiones de live coding.

# El sonido como datos

Antes de que empecemos detengámonos un momento para entender como trabajar con samples. Hasta ahora, hemos jugado con el potente sampler de Sonic Pi. Y si no, probémoslo ahora! Enciende tu Raspberry Pi, inicia Sonic Pi desde el menú Programación, escribe lo que viene a continuación en un buffer nuevo y pulsa el boton Run para poder escuchar un sonido de bateria pre-grabado:

```
sample :loop_amen
```

La grabación de un sonido es simplemente representada como datos - mogollón de números entre -1 y 1 que representan los picos y valles de la onda de sonido. Si reproducimos esos números en orden, oiremos el sonido original. Sin embargo, ¿qué nos impide reproducirlo en un orden diferentes para crear un sonido nuevo?

How are samples actually recorded? It's actually pretty simple once you understand the basic physics of sound. When you make a sound - for example by hitting a drum, the noise travels through the air in a similar fashion to how the surface of a lake ripples when you throw a pebble into it. When those ripples reach your ears, your eardrum moves sympathetically and converts those movements into the sound you hear. If we wish to record and play back the sound, we therefore need a way of capturing, storing and reproducing those ripples. One way is to use a microphone which acts like an eardrum and moves back and forth as the sound ripples hit it. The microphone then converts its position into a tiny electric signal which is then measured many times a second. These measurements are then represented as a series of numbers between -1 and 1.

Si fuéramos a visualizar en un gráfico el sonido sería un simple gráfico con información temporal en el eje x y con la posición del micrófono/altavoz como un valor entre -1 y 1 en el eje y. Puedes ver un ejemplo de un gráfico similar en la parte superior del diagrama.

# Reproducir una parte de un sample

So, how do we code Sonic Pi to play a sample back in a different order? To answer this question we need to take a look at the `start:` and `finish:` opts for `sample`. These let us control the start and finish positions of our playback of the numbers which represent the sound. The values for both of these opts are represented as a number between `0` and `1` where `0` represents the start of the sample and `1` is the end. So, to play the first half of the Amen Break, we just need to specify a `finish:` of `0.5`:

```
sample :loop_amen, finish: 0.5
```

Podemos añadir en el valor `start: para que toque incluso una sección más pequeña del sample

```
sample :loop_amen, start: 0.25, finish: 0.5
```

Para probar, puedes incluso poner el valor de la opción `finish:` *antes* del  `start:` y así tocará la sección macha atrás:

```
sample :loop_amen, start: 0.5, finish: 0.25
```

# Re-ordenando el playback de un sample

Ahora que sabemos que un sample es simplemente una lista de números que puede ser leída en cualquier orden y también cómo tocar una parte específica de un sample podemos empezar a pasarlo bien tocando un sample en el orden 'equivocado'.

![Amen Slices](../images/tutorial/articles/A.12-sample-slicing/amen_slice.png)

Let's take our Amen Break and chop it up into 8 equally-sized slices and then shuffle the pieces around. Take a look at the diagram: at the top A) represents the graph of our original sample data. Chopping it into 8 slices gives us B) - notice that we've given each slice a different colour to help distinguish them. You can see each slice's start and finish values at the top. Finally C) is one possible re-ordering of the slices. We can then play this back to create a new beat. Take a look at the code to do this:

```
live_loop :beat_slicer do
  slice_idx = rand_i(8)
  slice_size = 0.125
  s = slice_idx * slice_size
  f = s + slice_size
  sample :loop_amen, start: s, finish: f
  sleep sample_duration :loop_amen, start: s, finish: f
end
```

Elegimos un trozo al azar para tocar, que debe ser un numero aleatorio entre 0 y 7 (recuerda que tenemos que empezar a contar desde 0). Sonic Pi tiene una función adecuada para esto: `rand_i(8)`. Entonces guardamos el índice aleatorio del trozo en la variable `slice_idx`.
Definimos nuestro `slice_size` que es 1/8 o 0.125. El `slice_size` es necesario apara nosotros para convertir `slice_idx` en un valor entre 0 y 1 para que podamos usarlo en nuestra opcion `start:`.
Calculamos la posición de inicio `s` multiplicando `slice_idx` por `slice_size`.
Calculamos la posición final `f` añadiendo el  `slice_size` a la posición de inicio `s`.
Ahora podemos tocar el trozo de sample pasando los valores  `s` y `f` en las opciones `start:` y `finish:` de `sample`.
Antes de tocar el siguiente trozo necesitamos saber cuánto necesitamos esperar ( `sleep`). Esto debería de ser la duración del trozo del sample. Por suerte Sonic Pi nos ayuda con `sample_duration` que acepta las mismas opciones que  `sample` y simplemente nos devuelve la duración. Así pues, pasando a `sample_duration` nuestras opciones `start:` y `finish:`, podemos saber la duración de un solo trozo.
Ponemos todo este código en un 'live_loop' para continuar seleccionando aleatoriamente trozos nuevos que tocar.


## Bringing it all together

Combinemos ahora todo lo que hemos visto hasta ahora en un ejemplo final donde demostraremos cómo podemos tomar una estrategia similar para combinar aleatoriamente ritmos cortados con un ritmo de bajo para crear el inicio de un tema interesante. Ahora es tu turno - usa el código que hay aquí abajo como punto de inicio e intenta ver si puedes llevarlo en tu propia dirección para crear algo nuevo...

```
live_loop :sliced_amen do
  n = 8
  s =  line(0, 1, steps: n).choose
  f = s + (1.0 / n)
  sample :loop_amen, beat_stretch: 2, start: s, finish: f
  sleep 2.0  / n
end
live_loop :acid_bass do
  with_fx :reverb, room: 1, reps: 32, amp: 0.6 do
    tick
    n = (octs :e0, 3).look - (knit 0, 3 * 8, -4, 3 * 8).look
    co = rrand(70, 110)
    synth :beep, note: n + 36, release: 0.1, wave: 0, cutoff: co
    synth :tb303, note: n, release: 0.2, wave: 0, cutoff: co
    sleep (ring 0.125, 0.25).look
  end
end
```
